==> keystone-0.1.31.tgz: package/lib/fieldTypes/datetime.js <==
/*!
 * Module dependencies.
 */

var _ = require('underscore'),
	util = require('util'),
	moment = require('moment'),
	utils = require('../utils'),
	super_ = require('../field');

var parseFormats = ['YYYY-MM-DD', 'YYYY-MM-DD h:m:s a', 'YYYY-MM-DD h:m a', 'YYYY-MM-DD H:m:s', 'YYYY-MM-DD H:m'];

/**
 * DateTime FieldType Constructor
 * @extends Field
 * @api public
 */

function datetime(list, path, options) {
	
	this._nativeType = Date;
	this._underscoreMethods = ['format'];
	this.typeDescription = 'date and time';
	
	this._formatString = (options.format === false) ? false : (options.format || 'YYYY-MM-DD hh:mm:ss a');
	if (this._formatString && 'string' != typeof this._formatString) {
		throw new Error('FieldType.DateTime: options.format must be a string.');
	}
	
	datetime.super_.call(this, list, path, options);
	
	this.paths = {
		date: this._path.append('_date'),
		time: this._path.append('_time')
	};
	
};

/*!
 * Inherit from Field
 */
 
util.inherits(datetime, super_);


/**
 * Formats the field value
 * 
 * @api public
 */
datetime.prototype.format = function(item, format) {
	if (format || this._formatString) {
		return item.get(this.path) ? moment(item.get(this.path)).format(format || this._formatString) : '';
	} else {
		return item.get(this.path) || '';
	}
}


/**
 * Get the value from a data object; may be simple or a pair of fields
 * 
 * @api private
 */

datetime.prototype.getInputFromData = function(data) {
	if (this.paths.date in data && this.paths.time in data) {
		return (data[this.paths.date] + ' ' + data[this.paths.time]).trim();
	} else {
		return data[this.path];
	}
}


/**
 * Checks that a valid date has been provided in a data object
 * 
 * An empty value clears the stored value and is considered valid
 * 
 * @api public
 */

datetime.prototype.validateInput = function(data, required) {
	
	var newValue = moment(this.getInputFromData(data), parseFormats);
	
	if (required && (!newValue || !newValue.isValid())) {
		return false;
	} else if (newValue && !newValue.isValid()) {
		return false;
	} else {
		return true;
	}
	
}


/**
 * Updates the value for this field in the item from a data object
 * 
 * @api public
 */

datetime.prototype.updateItem = function(item, data) {
	
	if (!(item.path in data || (this.paths.date in data && this.paths.time in data)))
		return;
	
	var newValue = moment(this.getInputFromData(data), parseFormats);
	
	if (newValue && newValue.isValid()) {
		if (!item.get(this.path) || !newValue.isSame(item.get(this.path)))
			item.set(this.path, newValue.toDate());
	} else if (item.get(this.path)) {
		item.set(this.path, null);
	}
	
}


/*!
 * Export class
 */

exports = module.exports = datetime;

==> keystone-0.1.42.tgz: package/lib/fieldTypes/datetime.js <==
/*!
 * Module dependencies.
 */

var _ = require('underscore'),
	util = require('util'),
	moment = require('moment'),
	utils = require('keystone-utils'),
	super_ = require('../field');

var parseFormats = ['YYYY-MM-DD', 'YYYY-MM-DD h:m:s a', 'YYYY-MM-DD h:m a', 'YYYY-MM-DD H:m:s', 'YYYY-MM-DD H:m'];

/**
 * DateTime FieldType Constructor
 * @extends Field
 * @api public
 */

function datetime(list, path, options) {
	
	this._nativeType = Date;
	this._underscoreMethods = ['format'];
	this.typeDescription = 'date and time';
	
	this._formatString = (options.format === false) ? false : (options.format || 'Do MMM YYYY hh:mm:ss a');
	if (this._formatString && 'string' != typeof this._formatString) {
		throw new Error('FieldType.DateTime: options.format must be a string.');
	}
	
	datetime.super_.call(this, list, path, options);
	
	this.paths = {
		date: this._path.append('_date'),
		time: this._path.append('_time')
	};
	
};

/*!
 * Inherit from Field
 */
 
util.inherits(datetime, super_);


/**
 * Formats the field value
 * 
 * @api public
 */
datetime.prototype.format = function(item, format) {
	if (format || this._formatString) {
		return item.get(this.path) ? moment(item.get(this.path)).format(format || this._formatString) : '';
	} else {
		return item.get(this.path) || '';
	}
}


/**
 * Get the value from a data object; may be simple or a pair of fields
 * 
 * @api private
 */

datetime.prototype.getInputFromData = function(data) {
	if (this.paths.date in data && this.paths.time in data) {
		return (data[this.paths.date] + ' ' + data[this.paths.time]).trim();
	} else {
		return data[this.path];
	}
}


/**
 * Checks that a valid date has been provided in a data object
 * 
 * An empty value clears the stored value and is considered valid
 * 
 * @api public
 */

datetime.prototype.validateInput = function(data, required) {
	
	var newValue = moment(this.getInputFromData(data), parseFormats);
	
	if (required && (!newValue || !newValue.isValid())) {
		return false;
	} else if (newValue && !newValue.isValid()) {
		return false;
	} else {
		return true;
	}
	
}


/**
 * Updates the value for this field in the item from a data object
 * 
 * @api public
 */

datetime.prototype.updateItem = function(item, data) {
	
	if (!(item.path in data || (this.paths.date in data && this.paths.time in data)))
		return;
	
	var newValue = moment(this.getInputFromData(data), parseFormats);
	
	if (newValue && newValue.isValid()) {
		if (!item.get(this.path) || !newValue.isSame(item.get(this.path)))
			item.set(this.path, newValue.toDate());
	} else if (item.get(this.path)) {
		item.set(this.path, null);
	}
	
}


/*!
 * Export class
 */

exports = module.exports = datetime;
